(function($) {
	
	$.fn.spokenPlayer = function(option) {
		return this.each(function(i) {
			var player = new SpokenPlayer(this, option);
			$(this).data("spokenplayer", player);
		});
	}
	
	var SpokenPlayer = function(container, option) {
		var defaultOption = {
			onPlayEnded: null
		};
		
		this.container = container;
		this.newOption = $.extend({}, defaultOption, option || {});
		
		this.examId;
		this.events;
		this.type2events;
		this.spokenCallback;
		
		this.isplaying = false;
		
		this.player = $('<div class="spoken-player">' +
							'<div class="spoken-player-left">' +
								'<div class="spoken-player-button disabled"></div>' +
								'<div class="spoken-player-progressbar">' +
									'<div class="spoken-player-progressbar-highlight"></div>' +
								'</div>' +
								'<div class="spoken-player-time"><span id="currentTime">00:00:00</span> / <span id="totalTime">00:00:00</span></div>' +
							'</div>' +
							'<div class="spoken-player-right">' +
								'<div class="spoken-player-tip">已完成<span id="currentQue">0</span>题，共<span id="totalQue">0</span>题</div>' +
								'<div class="spoken-player-prev disabled"></div>' +
								'<div class="spoken-player-next disabled"></div>' +
							'</div>' +
							'<audio id="spokenAudio" hidden="hidden"></audio>' +
						'</div>');
						
		$(this.container).append(this.player);
		
		this.audio = this.player.find("audio").get(0);
		
		$(this.audio).on("canplay", {that: this}, function(evt) {
			// 开始按钮可点击
			evt.data.that.player.find(".spoken-player-button").removeClass("disabled");
		});
		
		$(this.audio).on("loadedmetadata", {that: this}, function(evt) {
			evt.data.that.player.find("#totalTime").text(formatPlayTime(evt.data.that.audio.duration));
		});
		
		$(this.audio).on("ended", {that: this}, function(evt) {
			var instance = evt.data.that;
			if (instance.newOption.onPlayEnded) {
				instance.newOption.onPlayEnded();
			}
		});
		
		var timeid;
		// 开始播放按钮
		this.player.find(".spoken-player-button").on("click", {that: this}, function(evt) {
			var player = evt.data.that.player;
			var audio = evt.data.that.audio;
			var events = evt.data.that.type2events;
			var instance = evt.data.that;
			
			if ($(this).hasClass("disabled")) {
				return;
			}
			if ($(this).hasClass("playing")) {
//				return;
				$(this).removeClass("playing");
				audio.pause();
				clearInterval(timeid);
			} else {
				$(this).addClass("playing");
				player.find(".spoken-player-prev").removeClass("disabled");
				player.find(".spoken-player-next").removeClass("disabled");
				audio.play();
				instance.isplaying = true;
				timeid = setInterval(function() {
					onplaying(player, audio, events, instance);
				}, 200);
			}
		});
		
		// 上一题
		this.player.find(".spoken-player-prev").on("click", {that: this}, function(evt) {
			var player = evt.data.that.player;
			var audio = evt.data.that.audio;
			var events = evt.data.that.type2events;
			var instance = evt.data.that;
			
			if (events == null || $(this).hasClass("disabled")) {
				return;
			}
			
			var currentIndex = parseInt(player.find("#currentQue").text()) - 1;
			if (currentIndex >= 0) {
				audio.currentTime = events[currentIndex].start;
			}
			
			if (!player.find(".spoken-player-button").hasClass("playing")) {
				player.find(".spoken-player-button").trigger("click");
			}
			
//			onplaying(player, audio, events, instance);
		});
		// 下一题
		this.player.find(".spoken-player-next").on("click", {that: this}, function(evt) {
			var player = evt.data.that.player;
			var audio = evt.data.that.audio;
			var events = evt.data.that.type2events;
			var instance = evt.data.that;
			
			if (events == null || $(this).hasClass("disabled")) {
				return;
			}
			
			var currentIndex = parseInt(player.find("#currentQue").text()) + 1;
			if (currentIndex < events.length) {
				audio.currentTime = events[currentIndex].start;
			}
			
			if (!player.find(".spoken-player-button").hasClass("playing")) {
				player.find(".spoken-player-button").trigger("click");
			}
			
//			onplaying(player, audio, events, instance);
		});
		
		this.init = function(eventData, audiourl) {
			this.restore();
			this.parseEventXML(eventData);
			this.initAudio(audiourl);
		}
		
		this.restore = function() {
			this.isplaying = false;
			this.player.find("#currentTime").text("00:00:00");
			this.player.find("#totalTime").text("00:00:00");
			this.player.find("#currentQue").text("0");
			this.player.find("#totalQue").text("0");
			this.player.find(".spoken-player-progressbar-highlight").width("0");
			this.player.find(".spoken-player-button").removeClass("playing").addClass("disabled");
			this.player.find(".spoken-player-prev").addClass("disabled");
			this.player.find(".spoken-player-next").addClass("disabled");
			this.audio.pause();
			$(this.audio).attr("src", null);
			this.events = null;
			this.type2events = null;
			this.spokenCallback = null;
			clearInterval(timeid);
		}
		
		this.parseEventXML = function(data) {
			this.events = new Array();
			this.type2events = new Array();
			
			var eventlist = $(data).find("time");
			
			for (var i = 0; i < eventlist.length; i++) {
				var e = eventlist[i];
				var evt = new SpokenEvent();
				evt.groupid = $(e).children("groupid").text();
				evt.titleid = $(e).children("titleid").text();
				evt.titletype = $(e).children("titletype").text();
				evt.start = $(e).children("start").text();
				evt.end = $(e).children("end").text();
				evt.eventtype = $(e).children("eventtype").text();
				evt.extend = $(e).children("extend").text();
				this.events.push(evt);
				
				if (evt.eventtype == "2") {
					this.type2events.push(evt);
				}
			}
			this.player.find("#totalQue").text(this.type2events.length);
		}
		
		this.initAudio = function(audiourl) {
			$(this.audio).attr("src", audiourl);
		}
		
		this.seekByGroupId = function(id) {
			if (!this.isplaying) {
				return;
			}
			var start = getStartByGroupid(id, this.type2events);
			this.audio.currentTime = start;
			
			if (!this.player.find(".spoken-player-button").hasClass("playing")) {
				this.player.find(".spoken-player-button").trigger("click");
			}
		}
		
		function onplaying(player, audio, events, that) {
			// 播放时间
			player.find("#currentTime").text(formatPlayTime(audio.currentTime));
			// 播放进度条
			player.find(".spoken-player-progressbar-highlight").width(audio.currentTime / audio.duration * 100 + "%");
			// 当前题目判断
			var currentQue;
			for (var i = 0; i < events.length; i++) {
				if (audio.currentTime > events[i].start) {
					currentQue = i;
				}
			}
			player.find("#currentQue").text(currentQue);
			if (that.spokenCallback) {
				that.spokenCallback(audio.currentTime);
			}
		}
		
		function getStartByGroupid(id, events) {
			if (events == null || events.length == 0) {
				return 0;
			}
			for (var i = 0; i < events.length; i++) {
				var evt = events[i];
				if (evt.groupid == id) {
					return evt.start;
				}
			}
			return 0;
		}
		
	}
	
	var SpokenEvent = function() {
		this.groupid;
		this.titleid;
		this.titletype;
		this.start;
		this.end;
		this.eventtype;
		this.extend;
	}
	
	function formatPlayTime(time) {
		var h = parseInt(time / 3600);
		var m = parseInt((time - h * 3600) / 60);
		var s = parseInt(time - h * 3600 - m * 60);
		return (h < 10 ? "0" + h : h) + ":" + (m < 10 ? "0" + m : m) + ":" + (s < 10 ? "0" + s : s);
	}
	
})(jQuery);
